@@ -1,7 +1,7 @@
 /*
  * string.h - Public API for Scheme strings
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2022  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -69,11 +69,20 @@
    As of 0.9.7, we switch to use ScmSmallInt for length and size, breaking
    the backward compatibility.
 */
+/* The 'index' slot may contain an index vector to realize O(1) random-access
+ * of the string.  Building index costs time and space, so it is only
+ * constructed when explicitly asked.  Srfi-135 (Immutable Texts) is
+ * really an immutable string with an indexed body.
+ * The user should treat index field as a opaque pointer.
+ * See priv/stringP.h for the details.
+ */
+
 typedef struct ScmStringBodyRec {
     u_long flags;
-    ScmSmallInt length;
-    ScmSmallInt size;
+    ScmSmallInt length;         /* in characters */
+    ScmSmallInt size;           /* in bytes */
     const char *start;
+    const void *index;
 } ScmStringBody;
 
 #if SIZEOF_LONG == 4
@@ -116,6 +125,7 @@ SCM_CLASS_DECL(Scm_StringClass);
 #define SCM_STRING_BODY_LENGTH(body)       ((body)->length)
 #define SCM_STRING_BODY_SIZE(body)         ((body)->size)
 #define SCM_STRING_BODY_START(body)        ((body)->start)
+#define SCM_STRING_BODY_END(body)          ((body)->start + (body)->size)
 #define SCM_STRING_BODY_FLAGS(body)        ((body)->flags)
 
 #define SCM_STRING_BODY_HAS_FLAG(body, flag) \
@@ -151,6 +161,36 @@ SCM_CLASS_DECL(Scm_StringClass);
 #define SCM_MAKSTR_IMMUTABLE   SCM_STRING_IMMUTABLE
 #define SCM_MAKSTR_COPYING     SCM_STRING_COPYING
 
+/*
+ * String cursors (srfi-130)
+ */
+
+/* If offset is small, we use immediate value (see gauche.h).
+ * When the offset doesn't fit, we fall back to ScmStringCursorLarge.
+ * The two types of cursors are handled transparently in string.c and
+ * users shouldn't worry about the distinction.
+ * The actual definition is in priv/stringP.h.
+ */
+typedef struct ScmStringCursorLargeRec ScmStringCursorLarge;
+
+SCM_CLASS_DECL(Scm_StringCursorClass);
+SCM_CLASS_DECL(Scm_StringCursorLargeClass);
+#define SCM_CLASS_STRING_CURSOR            (&Scm_StringCursorClass)
+#define SCM_CLASS_STRING_CURSOR_LARGE      (&Scm_StringCursorLargeClass)
+
+SCM_EXTERN int    Scm_StringCursorP(ScmObj obj);
+SCM_EXTERN ScmObj Scm_MakeStringCursorFromIndex(ScmString *src, ScmSmallInt index);
+SCM_EXTERN ScmObj Scm_MakeStringCursorEnd(ScmString *src);
+SCM_EXTERN ScmObj Scm_StringCursorIndex(ScmString *s, ScmObj sc);
+SCM_EXTERN ScmObj Scm_StringCursorStart(ScmString* s);
+SCM_EXTERN ScmObj Scm_StringCursorEnd(ScmString* s);
+SCM_EXTERN ScmObj Scm_StringCursorForward(ScmString* s, ScmObj cursor, int nchars);
+SCM_EXTERN ScmObj Scm_StringCursorBack(ScmString* s, ScmObj cursor, int nchars);
+SCM_EXTERN ScmChar Scm_StringRefCursor(ScmString* s, ScmObj sc, int range_error);
+SCM_EXTERN ScmObj Scm_SubstringCursor(ScmString *str, ScmObj start, ScmObj end);
+SCM_EXTERN int Scm_StringCursorCompare(ScmObj sc1, ScmObj sc2, int (*numcmp)(ScmObj, ScmObj));
+
+
 /*
  * Constructors
  */
@@ -159,7 +199,7 @@ SCM_EXTERN ScmObj  Scm_MakeString(const char *str,
                                   ScmSmallInt size, ScmSmallInt len,
                                   u_long flags);
 SCM_EXTERN ScmObj  Scm_MakeFillString(ScmSmallInt len, ScmChar fill);
-SCM_EXTERN ScmObj  Scm_CopyStringWithFlags(ScmString *str, 
+SCM_EXTERN ScmObj  Scm_CopyStringWithFlags(ScmString *str,
                                            u_long flags, u_long mask);
 
 #define SCM_MAKE_STR(cstr) \
@@ -269,7 +309,12 @@ enum {
     SCM_STRING_SCAN_AFTER,      /* return substring of s1 after s2 */
     SCM_STRING_SCAN_BEFORE2,    /* return substr of s1 before s2 and rest */
     SCM_STRING_SCAN_AFTER2,     /* return substr of s1 up to s2 and rest */
-    SCM_STRING_SCAN_BOTH        /* return substr of s1 before and after s2 */
+    SCM_STRING_SCAN_BOTH,       /* return substr of s1 before and after s2 */
+    SCM_STRING_SCAN_CURSOR,     /* return <string-cursor>
+                                   TRANSIENT: This may be more reasonable
+                                   to be after INDEX, but we have it here
+                                   until 1.0 to keep ABI compatibility */
+    SCM_STRING_SCAN_NUM_RETMODES
 };
 
 /*
@@ -281,6 +326,7 @@ SCM_EXTERN ScmSmallInt Scm_MBLen(const char *str, const char *stop);
 SCM_EXTERN const char *Scm_StringPosition(ScmString *str, ScmSmallInt k); /*DEPRECATED*/
 SCM_EXTERN const char *Scm_StringBodyPosition(const ScmStringBody *str, ScmSmallInt k);
 SCM_EXTERN ScmObj  Scm_MaybeSubstring(ScmString *x, ScmObj start, ScmObj end);
+SCM_EXTERN int     Scm_StringBodyFastIndexableP(const ScmStringBody *sb);
 
 /*
  * Static initializer
@@ -294,7 +340,7 @@ SCM_EXTERN ScmObj  Scm_MaybeSubstring(ScmString *x, ScmObj start, ScmObj end);
 
 #define SCM_STRING_CONST_INITIALIZER(str, len, siz)             \
     { { SCM_CLASS_STATIC_TAG(Scm_StringClass) }, NULL,          \
-      { SCM_STRING_IMMUTABLE|SCM_STRING_TERMINATED, (len), (siz), (str) } }
+    { SCM_STRING_IMMUTABLE|SCM_STRING_TERMINATED, (len), (siz), (str), NULL } }
 
 #define SCM_DEFINE_STRING_CONST(name, str, len, siz)            \
     ScmString name = SCM_STRING_CONST_INITIALIZER(str, len, siz)
@@ -333,8 +379,8 @@ SCM_EXTERN ScmSmallInt Scm_DStringSize(ScmDString *dstr);
 SCM_EXTERN ScmObj      Scm_DStringGet(ScmDString *dstr, u_long flags);
 SCM_EXTERN const char *Scm_DStringGetz(ScmDString *dstr);
 SCM_EXTERN void        Scm_DStringWeld(ScmDString *dstr);
-SCM_EXTERN const char *Scm_DStringPeek(ScmDString *dstr, 
-                                       ScmSmallInt *size, 
+SCM_EXTERN const char *Scm_DStringPeek(ScmDString *dstr,
+                                       ScmSmallInt *size,
                                        ScmSmallInt *len);
 SCM_EXTERN void        Scm_DStringPutz(ScmDString *dstr, const char *str,
                                        ScmSmallInt siz);
@@ -373,35 +419,4 @@ SCM_EXTERN void Scm__DStringRealloc(ScmDString *dstr, ScmSmallInt min_incr);
  */
 SCM_EXTERN char *Scm_StrdupPartial(const char *src, size_t size);
 
-/*
- * String pointers (WILL BE OBSOLETED)
- */
-
-/* Efficient way to access string from Scheme */
-typedef struct ScmStringPointerRec {
-    SCM_HEADER;
-    ScmSmallInt length;
-    ScmSmallInt size;
-    const char *start;
-    ScmSmallInt index;
-    const char *current;
-} ScmStringPointer;
-
-SCM_CLASS_DECL(Scm_StringPointerClass);
-#define SCM_CLASS_STRING_POINTER  (&Scm_StringPointerClass)
-#define SCM_STRING_POINTERP(obj)  SCM_XTYPEP(obj, SCM_CLASS_STRING_POINTER)
-#define SCM_STRING_POINTER(obj)   ((ScmStringPointer*)obj)
-
-SCM_EXTERN ScmObj Scm_MakeStringPointer(ScmString *src, ScmSmallInt index,
-                                        ScmSmallInt start, ScmSmallInt end);
-SCM_EXTERN ScmObj Scm_StringPointerRef(ScmStringPointer *sp);
-SCM_EXTERN ScmObj Scm_StringPointerNext(ScmStringPointer *sp);
-SCM_EXTERN ScmObj Scm_StringPointerPrev(ScmStringPointer *sp);
-SCM_EXTERN ScmObj Scm_StringPointerSet(ScmStringPointer *sp,
-                                       ScmSmallInt index);
-SCM_EXTERN ScmObj Scm_StringPointerSubstring(ScmStringPointer *sp, int beforep);
-SCM_EXTERN ScmObj Scm_StringPointerCopy(ScmStringPointer *sp);
-SCM_EXTERN void   Scm_StringPointerDump(ScmStringPointer *sp);
-
 #endif /* GAUCHE_STRING_H */
-
