@@ -17,10 +17,9 @@
 (define-module binary.pack
   (use srfi-11)   ;; let-values
   (use srfi-13)   ;; string library
-  (use srfi-14)   ;; char-set library
+  (use scheme.charset)
   (use text.parse)
   (use gauche.uvector)
-  (use gauche.parameter)
   (use binary.io)
   (export pack unpack unpack-skip make-packer))
 (select-module binary.pack)
@@ -601,7 +600,7 @@
                   (lambda ()
                     (let* ((actual-count (get-count (count-unpack)))
                            (bstr (read-block actual-count))
-                           (str (or (string-incomplete->complete! bstr) bstr))
+                           (str (or (string-incomplete->complete bstr) bstr))
                            (size (string-size str)))
                       (list
                        (if (< size actual-count)
@@ -610,7 +609,7 @@
                  (else
                   (lambda ()
                     (let* ((bstr (read-block count))
-                           (str (or (string-incomplete->complete! bstr) bstr))
+                           (str (or (string-incomplete->complete bstr) bstr))
                            (size (string-size str)))
                       (list
                        (if (< size count)
@@ -1050,5 +1049,3 @@
         (in (get-input-port rest)))
     (with-input-from-port in
       (cut packer 'skip))))
-
-
