@@ -1,7 +1,7 @@
 ;;;
 ;;; configure.scm - configuring Gauche extensions
 ;;;
-;;;   Copyright (c) 2013-2020  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2013-2022  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -109,7 +109,7 @@
           cf-check-type cf-type-available? cf-check-types
           cf-check-decl cf-decl-available? cf-check-decls
           cf-check-member cf-member-available? cf-check-members
-          
+
           cf-check-func cf-func-available? cf-check-funcs
           cf-check-lib cf-lib-available? cf-search-libs
           ))
@@ -165,7 +165,7 @@
 
 ;;;
 ;;; Basic APIs
-;;; 
+;;;
 
 ;; API
 ;; Like AC_MSG_*
@@ -415,7 +415,7 @@
           (exit 0)]
          ;; -x-includes
          ;; -x-libraries
-         
+
          ;; Deal with --enable|--disable|--with|--without options.
          ;; Note that parse-options split '=arg' part of --enable-*=arg or
          ;; --with-*=arg; if it is given, it will be in (car rest).
@@ -520,7 +520,7 @@
          (cf-msg-error "Unfulfilled dependency of package ~a: \
                         required to be ~s, but only found ~s."
                        (car req) (cadr req) vers)])))
-  
+
   (and-let* ([reqs (~ (current-package)'gpd'require)]
              [ (not (null? reqs)) ])
     (cf-msg-checking "package dependencies")
@@ -618,7 +618,7 @@
   (cf-path-prog 'GAUCHE_CESCONV  "gauche-cesconv")
 
   (cf-subst 'default_prefix (gauche-config "--prefix"))
- 
+
   (cf-subst 'GAUCHE_PKGINCDIR  (gauche-config "--pkgincdir"))
   (cf-subst 'GAUCHE_PKGLIBDIR  (gauche-config "--pkglibdir"))
   (cf-subst 'GAUCHE_PKGARCHDIR (gauche-config "--pkgarchdir"))
@@ -698,7 +698,7 @@
 
 ;; API
 ;; Temporarily replace cf subst value
-(define-syntax with-cf-subst 
+(define-syntax with-cf-subst
   (syntax-rules ()
     [(_ ((var val) ...) body ...)
      (let ([saves (map (cut cf$ <>) '(var ...))])
@@ -752,7 +752,7 @@
   (push! (arg-processors)
          (list feature 'enable help-string
                action-if-given action-if-not-given)))
-  
+
 ;; API
 (define (cf-arg-with package help-string
                      :optional (action-if-given (^[val] #f))
@@ -807,7 +807,14 @@
   (define base-substs (~ pa'substs))
   (define (make-defs)
     (if (null? (~ pa'config.h))
-      (string-join (dict-map (~ pa'defs) (^[k v] #"-D~|k|=~|v|")) " ")
+      (string-join
+       (dict-map (~ pa'defs)
+                 (^[k v]
+                   (let1 vv ($ regexp-replace-all* (x->string v)
+                               #/\\/ "\\\\\\\\"
+                               #/\"/ "\\\\\"")
+                     #"-D~|k|=~|vv|")))
+       " ")
       "-DHAVE_CONFIG_H"))
   (define (make-subst path-prefix)
     (receive (srcdir top_srcdir builddir top_builddir)
@@ -823,7 +830,7 @@
         (^[m]
           (let1 name (string->symbol (m 1))
             (or (dict-get substs name #f)
-                (begin (warn "@~a@ isn't substituted." name)
+                (begin (warn "@~a@ isn't substituted.\n" name)
                        #"@~|name|@")))))))
   ;; We use '/' in the replaced pathname even on Windows; that's what
   ;; autoconf-generated configure does, and it's less likely to confuse
@@ -849,7 +856,7 @@
                     builddir
                     (simplify-path+ (build-path builddir path-prefix)))
                   (simplify-path+ (build-path revpath tbuilddir)))))))
-                  
+
   (define (make-replace-1 output-file)
     (let1 subst (make-subst (sys-dirname (simplify-path+ output-file)))
       (^[line outp]
@@ -871,7 +878,7 @@
     (cf-subst 'prefix (cf$ 'default_prefix)))
   (when (equal? (cf$ 'exec_prefix) "NONE")
     (cf-subst 'exec_prefix "${prefix}"))
-  
+
   (dolist [f files]
     (let1 inf (build-path (cf$'srcdir) #"~|f|.in")
       (unless (file-is-readable? inf)
@@ -890,8 +897,32 @@
       (cf-msg-notice "configure: creating ~a" (car h))
       (file-filter-for-each (make-config.h) :input inf :output (car h)
                             :temporary-file #t :leave-unchanged #t)))
+
+  ;; Record output variables and definitions to config.log
+  (log-output-substs)
+  (log-output-defs)
   )
 
+(define (log-output-substs)
+  (log-format ".")
+  (log-format "## ----------------- ##")
+  (log-format "## Output variables. ##")
+  (log-format "## ----------------- ##")
+  (log-format ".")
+  (dolist [k (sort (hash-table-keys (~ (current-package)'substs)))]
+    (log-format "~a=~a" k (shell-escape-string
+                           (hash-table-get (~ (current-package)'substs) k)))))
+
+(define (log-output-defs)
+  (log-format ".")
+  (log-format "## ------------ ##")
+  (log-format "## Definitions. ##")
+  (log-format "## ------------ ##")
+  (log-format ".")
+  (dolist [k (sort (hash-table-keys (~ (current-package)'defs)))]
+    (log-format "#define ~a ~a" k
+                (or (hash-table-get (~ (current-package)'defs) k) "/**/"))))
+
 ;; API
 ;; Show definitions.
 (define (cf-show-substs :key (formatter (^[k v] (format #t "~16s ~s" k v))))
@@ -1087,8 +1118,9 @@
   (cf-arg-var 'CXX)
   (cf-arg-var 'CXXFLAGS)
   (cf-arg-var 'CCC)
-  (or (cf-ref 'CXX #f)
-      (and-let1 ccc (cf-ref 'CCC #f)
+  (or (not (string-null? (cf-ref 'CXX)))
+      (and-let* ([ccc (cf-ref 'CCC)]
+                 [ (not (string-null? ccc)) ])
         (cf-subst 'CXX ccc)
         #t)
       (cf-check-tool 'CXX compilers :default "g++"))
@@ -1132,7 +1164,8 @@
 ;; If SYM is already cf-subst'd, we don't do anything.
 (define (cf-check-prog sym prog-or-progs
                        :key (value #f) (default #f) (paths #f) (filter #f))
-  (unless (cf-have-subst? sym)
+  (unless (and (cf-have-subst? sym)
+               (not (string-null? (cf-ref sym))))
     (cf-msg-checking "for ~a" prog-or-progs)
     (if-let1 found (check-for-program (listify prog-or-progs)
                                       :paths paths :filter filter)
@@ -1146,7 +1179,8 @@
 ;; cf-path-prog works like AC_PATH_PROG and AC_PATH_PROGS.
 (define (cf-path-prog sym prog-or-progs
                       :key (value #f) (default #f) (paths #f) (filter #f))
-  (unless (cf-have-subst? sym)
+  (unless (and (cf-have-subst? sym)
+               (not (string-null? (cf-ref sym))))
     (cf-msg-checking "for ~a" prog-or-progs)
     (if-let1 found (check-for-program (listify prog-or-progs)
                                       :paths paths :filter filter)
@@ -1355,7 +1389,7 @@
 
 ;; Feature Test API
 ;; Like AC_CHECK_FUNCS
-(define (cf-check-funcs funcs :key (if-found identity) 
+(define (cf-check-funcs funcs :key (if-found identity)
                                    (if-not-found identity))
   (dolist [f funcs]
     (if (cf-check-func f)
@@ -1396,17 +1430,17 @@
   (when libname
     (cf-subst-prepend 'LIBS #"-l~|libname|"))
   #t)
-   
+
 ;; Feature test API
 ;; Like AC_CHECK_LIBS
 (define (cf-search-libs fn libs
                         :key (other-libs '())
-                             (if-found default-lib-search-found) 
+                             (if-found default-lib-search-found)
                              (if-not-found default-lib-not-found))
   (let ([includes (cf-includes-default)]
         [xlibs #"~(string-join other-libs \" \") ~(cf$'LIBS)"])
     (define (try lib)
-      (with-cf-subst 
+      (with-cf-subst
        ([LIBS (if (eq? lib 'none) xlibs #"-l~|lib| ~xlibs")])
        (cf-try-compile-and-link includes
                                 (format "extern void ~a(); ~a();"
